package org.usfirst.frc.team1403.robot;

import edu.wpi.first.wpilibj.command.Command;
import edu.wpi.first.wpilibj.TimedRobot;
import edu.wpi.first.wpilibj.Timer;
import edu.wpi.first.wpilibj.command.Scheduler;
import edu.wpi.first.wpilibj.smartdashboard.SendableChooser;
import edu.wpi.first.wpilibj.smartdashboard.SmartDashboard;

import org.usfirst.frc.team1403.robot.subsystems.Climber;
import org.usfirst.frc.team1403.robot.subsystems.DriveTrain;

import cougarecho.record.Recorder;

/**
 * The VM is configured to automatically run this class, and to call the
 * functions corresponding to each mode, as described in the IterativeRobot
 * documentation. If you change the name of this class or the package after
 * creating this project, you must also update the manifest file in the resource
 * directory.
 */
public class Robot extends TimedRobot {
	private int numpaths;
	private String path;
	public static OI m_oi;
	public static DriveTrain drivetrain;
	public static Climber climber;
	public static Recorder recorder;
	public static boolean record;
	public static boolean store;

	Command autonomousCommand;
	SendableChooser<Command> chooser = new SendableChooser<>();
	
	/**
	 * This function is run when the robot is first started up and should be
	 * used for any initialization code.
	 */
	@Override
	public void robotInit() {
		recorder = new Recorder(10000); //10000 = max arr size
		drivetrain = new DriveTrain();
		climber = new Climber();
		m_oi = new OI();
		
		numpaths = 0;
		init();
		recorder.setCurrentWritefile(1);
		recorder.setCurrentReadfile(0);
		Recorder.initWriter();
		Recorder.initReader();
	}

	/**
	 * This function is called once each time the robot enters Disabled mode.
	 * You can use it to reset any subsystem information you want to clear when
	 * the robot is disabled.
	 */
	@Override
	public void disabledInit() {
		//recorder.doneWrite();
	}

	@Override
	public void disabledPeriodic() {
		Scheduler.getInstance().run();
	}		

	/**
	 * This autonomous (along with the chooser code above) shows how to select
	 * between different autonomous modes using the dashboard. The sendable
	 * chooser code works with the Java SmartDashboard. If you prefer the
	 * LabVIEW Dashboard, remove all of the chooser code and uncomment the
	 * getString code to get the auto name from the text box below the Gyro
	 *
	 * You can add additional auto modes by adding additional commands to the
	 * chooser code above (like the commented example) or additional comparisons
	 * to the switch structure below with additional strings & commands.
	 */
	@Override
	public void autonomousInit() {
		recorder.resetReadings();
		recorder.storeReadings();
		autonomousCommand = chooser.getSelected();
		if (autonomousCommand != null)
			autonomousCommand.start();
	}

	/**
	 * This function is called periodically during autonomous
	 */
	@Override
	public void autonomousPeriodic() {
		if(recorder.hasNextLine()) {			
			DriveTrain.setSpeed(DriveTrain.m6, recorder.getReading("DriveTrain L"));
			DriveTrain.setSpeed(DriveTrain.m2, recorder.getReading("DriveTrain R"));
			climber.climb(recorder.getReading("Climber"));
			//climber.climb(recorder.getInterpReading("Climber")); //interpolated reading
			Timer.delay(0.001);
		} else {
			DriveTrain.setSpeed(DriveTrain.m6, 0);
			DriveTrain.setSpeed(DriveTrain.m2, 0);
		}
		Scheduler.getInstance().run();
	}

	@Override
	public void teleopInit() {
		// This makes sure that the autonomous stops running when
		// teleop starts running. If you want the autonomous to
		// continue until interrupted by another command, remove
		// this line or comment it out.
		if (autonomousCommand != null)
			autonomousCommand.cancel();
	}

	/**
	 * This function is called periodically during operator control
	 */
	@Override
	public void teleopPeriodic() {
		Scheduler.getInstance().run();
		if(Recorder.isRecording) {
			recorder.addReading("DriveTrain L", drivetrain.getRawAxisLeft);
			recorder.addReading("DriveTrain R", drivetrain.getRawAxisRight);
			recorder.addReading("Climber", climber.climbReading);
			recorder.initNextReading();
		} else if (Recorder.isStoring()) {
			recorder.storeWritings();
		}
	}

	/**
	 * This function is called periodically during test mode
	 */
	@Override
	public void testPeriodic() {}
	
	public void init() {
		//File Select Menu
		path = new String("/home/lvuser/file1.txt");
		recorder.addFileSelect(numpaths, path);
		SmartDashboard.putString(Integer.toString(numpaths), path);
		++numpaths;
		path = new String("/home/lvuser/file2.txt");
		recorder.addFileSelect(numpaths, path);
		SmartDashboard.putString(Integer.toString(numpaths), path);
		++numpaths;
	}
}
